@@ -1,11 +1,10 @@
 __name__ = "socket_api2"
-__version__ = "0.0.5"
+__version__ = "0.0.7"
 
 __author__ = "Da4ndo"
 __discord__ = "Da4ndo#0934"
 __youtube__ = "https://www.youtube.com/channel/UCdhZa-JISiqwd913nhB8cAw?"
 __github__ = "https://github.com/Mesteri05"
-__package__ = "None"
 
 import socket
 from pyngrok import ngrok
@@ -251,6 +250,7 @@ class Server():
         self.client_timeout = client_timeout
 
         self.decorator_functions_call = []
+        self.decorator_client_disconnect = []
 
         self.clients = []
 
@@ -291,6 +291,8 @@ class Server():
             if not client.check_connection():
                 self.clients.remove(client)
                 if self.console_output: outstr("DEBUG", f"{client.ip} is no longer connected. (Disconnect)")
+                for func in self.decorator_functions_call:
+                    _thread.start_new_thread(func, (client,))
 
         while True:
             time.sleep(10)
@@ -303,7 +305,11 @@ class Server():
     
     @property
     def connected_clients(self):
-        return self.IP
+        return self.clients
+    
+    @property
+    def socket_object(self):
+        return self.server
 
     def sendall(self, msg):
         if isinstance(msg, str):
@@ -315,13 +321,21 @@ class Server():
 
         self.server.sendall(msg)
     
-    def when_client_connect(self, *args, **kwargs):
+    def on_client_connect(self, *args, **kwargs):
         def decorator(func):
             if func not in self.decorator_functions_call:
                 self.decorator_functions_call.append(func)
             return func
 
         return decorator
+    
+    def on_client_disconnect(self, *args, **kwargs):
+        def decorator(func):
+            if func not in self.decorator_client_disconnect:
+                self.decorator_client_disconnect.append(func)
+            return func
+
+        return decorator
 
 class MultiClient():
     def  __init__(self, target_ip:str, target_port:int, console_output:bool=True, timeout:int=None, reconnect:bool=True):
@@ -337,6 +351,10 @@ class MultiClient():
 
         self.client.settimeout(timeout)
     
+    @property
+    def socket_object(self):
+        return self.client
+
     def connect(self):
         self.client.connect((self.ip, self.port))
         if self.console_output: outstr("INFO", f"Connected to {self.ip}:{self.port}")
@@ -395,6 +413,9 @@ class MultiClient():
         except Exception:
             raise Exception
     
+    def sendfile(self, file, offset=0, count=None):
+        self.client.sendfile(file, offset, count)
+    
     def recv(self, buffer:int):
         self.check_connection()
 
